#!/usr/bin/python
# -*- coding: iso-8859-15 -*-
## ----------------------------------
## TP cc
## Etudiant 1: 
## Etudiant 2: 
## ----------------------------------
import string
from collections import deque
import sys 


##  fonction de chargement de lexique
##  le lexique est un sous-ensemble de mots du fichier en entrée 'fichier'
##  tel que les mots extraits ont la même longueur que le mot en entrée 'mot'
##  input : 'fichier' et 'mot'
##  output : la fonction retourne le lexique de mots sous forme de liste de string
##  (un set est une liste qui garantit qu'il n'y a pas de doublon)
def lexique_meme_taille(fichier, mot):
	return set(m for m in open(fichier).read().splitlines() if len(m) == len(mot))
	
	
##  Autre fonction de chargement de lexique
##  le lexique est l'ensemble de mots du fichier en entrée 'fichier'
##  la fonction retourne le lexique de mots sous forme de liste de string
##  input : 'fichier'
##  output : la fonction retourne le lexique de mots sous forme de liste de string
## (un set est une liste qui garantit qu'il n'y a pas de doublon)
def lexique_complet(fichier):
	return set(m for m in open(fichier).read().splitlines())
 
##  fonction qui génère les suivants potentiels dans la chaine
##  ici, la règle appliquée est celle qui autorise à chaîner les
##  les mots qui ont une lettre de différence.
##  Cette fonction génère tous les mots possibles, en remplacant la première
##  par les 25 autres de l'alphabet, puis la deuxième, etc.
##  pour un mot de 4 lettres, on va donc générer 100 mots, dont une bonne part
##  seront fictifs.
##  input : le mot à qui l'on veut trouver les suivants potentiels
##  ouput : une liste de mots potentiels 
def suivant_potentiel(mot_pere):
	combinaisons = set()
	# indice qui va servir de curseur pour se déplacer de lettre en lettre dans le mot_pere. 
	for i in range(len(mot_pere)):
		# on parcourt toutes les lettres de l'alphabet en minuscule, et pour chacune on va générer un mot nouveau
		for l in string.ascii_lowercase:
			# on vérifie que la lettre remplacante n'est pas identique à la lettre courante en position i
			if l != mot_pere[i]:
				# et on remplace la lettre courante en position i par l pour les 25 lettres différentes
				combinaisons.add(mot_pere[:i] + l + mot_pere[i+1:])
	return combinaisons
  
##  fonction qui sélectionne les vrais mots, i.e. ceux du lexique.txt, parmi les mots générés
##  par la fonction suivant_potentiel(mot_pere), mots pour la plupart fictifs.
##  fait l'intersection des mots générés (ensemble des suivants potentiels) et des vrais mots.
##  input : le lexique de "vrais" mots
##  ouput : un dict (object python) donnant pour un mot, les mots suivants présents dans le lexique 
def dict_graph(dico):
	def vrais_suivants(mot_pere):
		return dico & suivant_potentiel(mot_pere)
	
	# genere un dictionnaire python avec pour chaque mot, les candidat potentiels valides que l'on pourra chainer	  
	return {mot_pere: vrais_suivants(mot_pere) for mot_pere in dico}
  

 

## --------------------  MAIN -------------------- ##

## Interaction avec l'utilisateur
## L'utilisateur doit entrer deux mots   
mot1 = raw_input("Entrez le mot 1 puis touche <Entree> : ")
mot2 = raw_input("Entrez le mot 2 puis touche <Entree> : ")
#mot1 = "rain"
#mot2 = "zyme"

## Chargement du lexique
mots = lexique_meme_taille("lexique.txt",mot1)
#mots = lexique_complet("lexique.txt")

print "Le lexique extrait contient " + str(len(mots)) + " mots.\n"
  
if mot1 not in mots or mot2 not in mots:
	print('L\'un au moins des mots entrés est inconnu.')
	sys.exit(0)
 
## On crée un graphe de mots
## Sauriez-vous le représenter ?
graphe=dict_graph(mots) 

print "Les voisins du mot 1 sont :" + str(graphe[mot1]) + "\n"
  
## C'est à vous !
## Que fait-on avec notre graphe de mots ?
## Le résultat doit être une liste de mots
## Pour le moment result est vide, à vous de
## le remplir.

result=[]

## Le résultat attendu est une liste de mots
print("Enchainement des mots trouvé")
if result is not None:
	print('\n'.join(result))
else:
	print('Aucune transformation trouvée pour passer de mot1 à mot2.')
